@@ -1,5 +1,4 @@
 import logging
-from logging.config import fileConfig
 
 from alembic import context
 from sqlalchemy import engine_from_config, pool
@@ -14,7 +13,8 @@ config = context.config
 
 # Interpret the config file for Python logging.
 # This line sets up loggers basically.
-fileConfig(config.config_file_name)  # type: ignore
+# Dont run because it removes all other loggers
+# fileConfig(config.config_file_name, disable_existing_loggers=False)  # type: ignore
 logging.getLogger("alembic").setLevel(logging.CRITICAL)
 
 # add your model's MetaData object here
